@@ -89,24 +89,6 @@ static bool find (char *arg) __attribute_warn_unused_result__;
 static bool process_all_startpoints (int argc, char *argv[]) __attribute_warn_unused_result__;
 
 
-int
-get_current_dirfd (void)
-{
-  if (ftsoptions & FTS_CWDFD)
-    {
-      assert (curr_fd != -1);
-      assert ( (AT_FDCWD == curr_fd) || (curr_fd >= 0) );
-
-      if (AT_FDCWD == curr_fd)
-	return starting_desc;
-      else
-	return curr_fd;
-    }
-  else
-    {
-      return AT_FDCWD;
-    }
-}
 
 static void
 left_dir (void)
@@ -299,15 +281,6 @@ symlink_loop (const char *name)
 }
 
 
-static int
-complete_execdirs_cb (void *context)
-{
-  (void) context;
-  /* By the tme this callback is called, the current directory is correct. */
-  complete_pending_execdirs (AT_FDCWD);
-  return 0;
-}
-
 static void
 show_outstanding_execdirs (FILE *fp)
 {
@@ -540,24 +513,6 @@ consider_visiting (FTS *p, FTSENT *ent)
       visit (p, ent, &statbuf);
     }
 
-  /* XXX: if we allow a build-up of pending arguments for "-execdir foo {} +"
-   * we need to execute them in the same directory as we found the item.
-   * If we are trying to do "find a -execdir echo {} +", we will need to
-   * echo
-   *      a while in the original working directory
-   *      b while in a
-   *      c while in b (just before leaving b)
-   *
-   * These restrictions are hard to satisfy while using fts().   The reason is
-   * that it doesn't tell us just before we leave a directory.  For the moment,
-   * we punt and don't allow the arguments to build up.
-   */
-  if (state.execdirs_outstanding)
-    {
-      show_outstanding_execdirs (stderr);
-      run_in_dir (p->fts_cwd_fd, complete_execdirs_cb, NULL);
-    }
-
   if (ent->fts_info == FTS_DP)
     {
       /* we're leaving a directory. */
@@ -607,8 +562,27 @@ find (char *arg)
     }
   else
     {
+      int level = INT_MIN;
+
       while ( (ent=fts_read (p)) != NULL )
 	{
+	  if (state.execdirs_outstanding)
+	    {
+	      /* If we changed level, perform any outstanding
+	       * execdirs.  If we see a sequence of directory entries
+	       * like this: fffdfffdfff, we could build a command line
+	       * of 9 files, but this simple-minded implementation
+	       * builds a command line for only 3 files at a time
+	       * (since fts descends into the directories).
+	       */
+	      if ((int)ent->fts_level != level)
+		{
+		  show_outstanding_execdirs (stderr);
+		  complete_pending_execdirs ();
+		}
+	    }
+	  level = (int)ent->fts_level;
+
 	  state.already_issued_stat_error_msg = false;
 	  state.have_stat = false;
 	  state.have_type = !!ent->fts_statp->st_mode;
@@ -674,6 +648,8 @@ main (int argc, char **argv)
   else
     set_program_name ("find");
 
+  record_initial_cwd ();
+
   state.already_issued_stat_error_msg = false;
   state.exit_status = 0;
   state.execdirs_outstanding = false;
@@ -740,23 +716,6 @@ main (int argc, char **argv)
     }
 
 
-  starting_desc = open_cloexec (".", O_RDONLY
-#if defined O_LARGEFILE
-				|O_LARGEFILE
-#endif
-				);
-  if (0 <= starting_desc && fchdir (starting_desc) != 0)
-    {
-      close (starting_desc);
-      starting_desc = -1;
-    }
-  if (starting_desc < 0)
-    {
-      starting_dir = xgetcwd ();
-      if (! starting_dir)
-	error (EXIT_FAILURE, errno, _("cannot get current directory"));
-    }
-
   /* process_all_startpoints processes the starting points named on
    * the command line.  A false return value from it means that we
    * failed to restore the original context.  That means it would not
