@@ -48,6 +48,8 @@
 #include "verify.h"
 #include "fdleak.h"
 #include "areadlink.h"
+#include "cloexec.h"
+#include "save-cwd.h"
 
 #include <selinux/selinux.h>
 
@@ -163,10 +165,10 @@
 #undef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 
-static bool match_lname PARAMS((const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, bool ignore_case));
+static bool match_lname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, bool ignore_case);
 
-static char *format_date PARAMS((struct timespec ts, int kind));
-static char *ctime_format PARAMS((struct timespec ts));
+static char *format_date (struct timespec ts, int kind);
+static char *ctime_format (struct timespec ts);
 
 #ifdef	DEBUG
 struct pred_assoc
@@ -504,17 +506,113 @@ pred_empty (const char *pathname, struct stat *stat_buf, struct predicate *pred_
     return (false);
 }
 
+
+/* Initialise exec->wd_for_exec.
+
+   We save in exec->wd_for_exec the directory whose path relative to
+   cwd_df is dir.
+ */
 static bool
-new_impl_pred_exec (int dir_fd, const char *pathname,
-		    struct stat *stat_buf,
-		    struct predicate *pred_ptr,
-		    const char *prefix, size_t pfxlen)
+initialise_wd_for_exec (struct exec_val *execp, int cwd_fd, const char *dir)
+{
+  execp->wd_for_exec = xmalloc (sizeof (*execp->wd_for_exec));
+  execp->wd_for_exec->name = NULL;
+  execp->wd_for_exec->desc = openat (cwd_fd, dir, O_RDONLY);
+  if (execp->wd_for_exec->desc < 0)
+    return false;
+  set_cloexec_flag (execp->wd_for_exec->desc, true);
+  return true;
+}
+
+
+static bool
+record_exec_dir (struct exec_val *execp)
+{
+  if (!execp->wd_for_exec)
+    {
+      /* working directory not already known, so must be a *dir variant,
+	 and this must be the first arg we added.   However, this may
+	 be -execdir foo {} \; (i.e. not multiple).  */
+      assert (!execp->state.todo);
+
+      /* Record the WD. If we're using -L or fts chooses to do so for
+	 any other reason, state.cwd_dir_fd may in fact not be the
+	 directory containing the target file.  When this happens,
+	 rel_path will contain directory components (since it is the
+	 path from state.cwd_dir_fd to the target file).
+
+	 We deal with this by extracting any directory part and using
+	 that to adjust what goes into execp->wd_for_exec.
+      */
+      if (strchr (state.rel_pathname, '/'))
+	{
+	  char *dir = mdir_name (state.rel_pathname);
+	  bool result = initialise_wd_for_exec (execp, state.cwd_dir_fd, dir);
+	  free (dir);
+	  return result;
+	}
+      else
+	{
+	  return initialise_wd_for_exec (execp, state.cwd_dir_fd, ".");
+	}
+    }
+  return true;
+}
+
+
+static bool
+impl_pred_exec (const char *pathname,
+		struct stat *stat_buf,
+		struct predicate *pred_ptr)
 {
   struct exec_val *execp = &pred_ptr->args.exec_vec;
-  size_t len = strlen (pathname);
+  char *target;
+  bool result;
+  const bool local = is_exec_in_local_dir (pred_ptr->pred_func);
+  char *prefix;
+  size_t pfxlen;
 
   (void) stat_buf;
-  execp->dir_fd = dir_fd;
+  if (local)
+    {
+      /* For -execdir/-okdir predicates, the parser did not fill in
+	 the wd_for_exec member of sturct exec_val.  So for those
+	 predicates, we do so now.
+      */
+      if (!record_exec_dir (execp))
+	{
+	  error (EXIT_FAILURE, errno,
+		 _("Failed to save working directory in order to "
+		   "run a command on %s"),
+		 safely_quote_err_filename (0, pathname));
+	  /*NOTREACHED*/
+	}
+      target = base_name (state.rel_pathname);
+      if ('/' == target[0])
+	{
+	  /* find / execdir ls -d {} \; */
+	  prefix = NULL;
+	  pfxlen = 0;
+	}
+      else
+	{
+	  prefix = "./";
+	  pfxlen = 2u;
+	}
+    }
+  else
+    {
+      /* For the others (-exec, -ok), the parser should
+	 have set wd_for_exec to initial_wd, indicating
+	 that the exec should take place from find's initial
+	 working directory.
+      */
+      assert (execp->wd_for_exec == initial_wd);
+      target = pathname;
+      prefix = NULL;
+      pfxlen = 0u;
+    }
+
   if (execp->multiple)
     {
       /* Push the argument onto the current list.
@@ -523,7 +621,7 @@ new_impl_pred_exec (int dir_fd, const char *pathname,
        */
       bc_push_arg (&execp->ctl,
 		   &execp->state,
-		   pathname, len+1,
+		   target, strlen (target)+1,
 		   prefix, pfxlen,
 		   0);
 
@@ -533,7 +631,7 @@ new_impl_pred_exec (int dir_fd, const char *pathname,
       /* POSIX: If the primary expression is punctuated by a plus
        * sign, the primary shall always evaluate as true
        */
-      return true;
+      result = true;
     }
   else
     {
@@ -546,7 +644,7 @@ new_impl_pred_exec (int dir_fd, const char *pathname,
 			execp->replace_vec[i],
 			strlen (execp->replace_vec[i]),
 			prefix, pfxlen,
-			pathname, len,
+			target, strlen (target),
 			0);
 	}
 
@@ -555,33 +653,35 @@ new_impl_pred_exec (int dir_fd, const char *pathname,
       if (WIFEXITED(execp->last_child_status))
 	{
 	  if (0 == WEXITSTATUS(execp->last_child_status))
-	    return true;	/* The child succeeded. */
+	    result = true;	/* The child succeeded. */
 	  else
-	    return false;
+	    result = false;
 	}
       else
 	{
-	  return false;
+	  result = false;
 	}
     }
+  if (target != pathname)
+    {
+      assert (local);
+      free (target);
+    }
+  return result;
 }
 
 
 bool
 pred_exec (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  return new_impl_pred_exec (get_start_dirfd(),
-			     pathname, stat_buf, pred_ptr, NULL, 0);
+  return impl_pred_exec (pathname, stat_buf, pred_ptr);
 }
 
 bool
 pred_execdir (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-   const char *prefix = (state.rel_pathname[0] == '/') ? NULL : "./";
    (void) &pathname;
-   return new_impl_pred_exec (get_current_dirfd (),
-			      state.rel_pathname, stat_buf, pred_ptr,
-			      prefix, (prefix ? 2 : 0));
+   return impl_pred_exec (state.rel_pathname, stat_buf, pred_ptr);
 }
 
 bool
@@ -1482,8 +1582,7 @@ bool
 pred_ok (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   if (is_ok (pred_ptr->args.exec_vec.replace_vec[0], pathname))
-    return new_impl_pred_exec (get_start_dirfd (),
-			       pathname, stat_buf, pred_ptr, NULL, 0);
+    return impl_pred_exec (pathname, stat_buf, pred_ptr);
   else
     return false;
 }
@@ -1491,11 +1590,8 @@ pred_ok (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr
 bool
 pred_okdir (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  const char *prefix = (state.rel_pathname[0] == '/') ? NULL : "./";
   if (is_ok (pred_ptr->args.exec_vec.replace_vec[0], pathname))
-    return new_impl_pred_exec (get_current_dirfd (),
-			       state.rel_pathname, stat_buf, pred_ptr,
-			       prefix, (prefix ? 2 : 0));
+    return impl_pred_exec (state.rel_pathname, stat_buf, pred_ptr);
   else
     return false;
 }
@@ -1570,35 +1666,6 @@ pred_perm (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
 }
 
 
-struct access_check_args
-{
-  const char *filename;
-  int access_type;
-  int cb_errno;
-};
-
-
-static int
-access_callback (void *context)
-{
-  int rv;
-  struct access_check_args *args = context;
-  if ((rv = access (args->filename, args->access_type)) < 0)
-    args->cb_errno = errno;
-  return rv;
-}
-
-static int
-can_access (int access_type)
-{
-  struct access_check_args args;
-  args.filename = state.rel_pathname;
-  args.access_type = access_type;
-  args.cb_errno = 0;
-  return 0 == run_in_dir (state.cwd_dir_fd, access_callback, &args);
-}
-
-
 bool
 pred_executable (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
@@ -1606,7 +1673,8 @@ pred_executable (const char *pathname, struct stat *stat_buf, struct predicate *
   (void) stat_buf;
   (void) pred_ptr;
 
-  return can_access (X_OK);
+  /* As for access, the check is performed with the real user id. */
+  return 0 == faccessat (state.cwd_dir_fd, state.rel_pathname, X_OK, 0);
 }
 
 bool
@@ -1616,7 +1684,8 @@ pred_readable (const char *pathname, struct stat *stat_buf, struct predicate *pr
   (void) stat_buf;
   (void) pred_ptr;
 
-  return can_access (R_OK);
+  /* As for access, the check is performed with the real user id. */
+  return 0 == faccessat (state.cwd_dir_fd, state.rel_pathname, R_OK, 0);
 }
 
 bool
@@ -1626,7 +1695,8 @@ pred_writable (const char *pathname, struct stat *stat_buf, struct predicate *pr
   (void) stat_buf;
   (void) pred_ptr;
 
-  return can_access (W_OK);
+  /* As for access, the check is performed with the real user id. */
+  return 0 == faccessat (state.cwd_dir_fd, state.rel_pathname, W_OK, 0);
 }
 
 bool
@@ -1955,7 +2025,7 @@ pred_context (const char *pathname, struct stat *stat_buf,
 
 
 static bool
-prep_child_for_exec (bool close_stdin, int dir_fd)
+prep_child_for_exec (bool close_stdin, const struct saved_cwd *wd)
 {
   bool ok = true;
   if (close_stdin)
@@ -1991,17 +2061,10 @@ prep_child_for_exec (bool close_stdin, int dir_fd)
    * announcement of a call to stat() anyway, as we're about to exec
    * something.
    */
-  if (dir_fd != AT_FDCWD)
+  if (0 != restore_cwd (wd))
     {
-      assert (dir_fd >= 0);
-      if (0 != fchdir (dir_fd))
-	{
-	  /* If we cannot execute our command in the correct directory,
-	   * we should not execute it at all.
-	   */
-	  error (0, errno, _("Failed to change directory"));
-	  ok = false;
-	}
+      error (0, errno, _("Failed to change directory"));
+      ok = false;
     }
   return ok;
 }
@@ -2018,12 +2081,6 @@ launch (struct buildcmd_control *ctl, void *usercontext, int argc, char **argv)
   static int first_time = 1;
   struct exec_val *execp = usercontext;
 
-  if (!execp->use_current_dir)
-    {
-      assert (starting_desc >= 0);
-      assert (execp->dir_fd == starting_desc);
-    }
-
   /* Make sure output of command doesn't get mixed with find output. */
   fflush (stdout);
   fflush (stderr);
@@ -2041,8 +2098,8 @@ launch (struct buildcmd_control *ctl, void *usercontext, int argc, char **argv)
   if (child_pid == 0)
     {
       /* We are the child. */
-      assert (starting_desc >= 0);
-      if (!prep_child_for_exec (execp->close_stdin, execp->dir_fd))
+      assert (NULL != execp->wd_for_exec);
+      if (!prep_child_for_exec (execp->close_stdin, execp->wd_for_exec))
 	{
 	  _exit (1);
 	}
