@@ -57,12 +57,12 @@ static size_t num_start_points = 0;
 
 
 
-static struct predicate *scan_rest PARAMS((struct predicate **input,
-					   struct predicate *head,
-					   short int prev_prec));
-static void merge_pred PARAMS((struct predicate *beg_list, struct predicate *end_list, struct predicate **last_p));
-static struct predicate *set_new_parent PARAMS((struct predicate *curr, enum predicate_precedence high_prec, struct predicate **prevp));
-static const char *cost_name PARAMS((enum EvaluationCost cost));
+static struct predicate *scan_rest (struct predicate **input,
+				    struct predicate *head,
+				    short int prev_prec);
+static void merge_pred (struct predicate *beg_list, struct predicate *end_list, struct predicate **last_p);
+static struct predicate *set_new_parent (struct predicate *curr, enum predicate_precedence high_prec, struct predicate **prevp);
+static const char *cost_name (enum EvaluationCost cost);
 
 
 /* Return true if the indicated path name is a start
@@ -345,17 +345,6 @@ predlist_insert (struct predlist *list,
   struct predicate **insertpos = &(list->head);
 
   *pprev = curr->pred_left;
-  if (options.optimisation_level > 2)
-    {
-      /* Insert the new node in the list after any other entries which
-       * are more selective.
-       */
-      if (0)
-	while ( (*insertpos) && ((*insertpos)->est_success_rate < curr->est_success_rate) )
-	  {
-	    insertpos = &((*insertpos)->pred_left);
-	  }
-    }
   curr->pred_left = (*insertpos);
   (*insertpos) = curr;
   if (NULL == list->tail)
@@ -1100,7 +1089,7 @@ get_pred_cost (const struct predicate *p)
 			     cost_table_comparison))
 	    {
 	      error (EXIT_FAILURE, 0,
-		     "Failed to sort the costlookup array (indirect).");
+		     "failed to sort the costlookup array");
 	    }
 	  pred_table_sorted = 1;
 	}
@@ -1115,7 +1104,16 @@ get_pred_cost (const struct predicate *p)
 	}
       else
 	{
-	  error(0, 0, "warning: no cost entry for predicate %s", p->p_name);
+	  /* This message indicates a bug.  If we issue the message, we
+	     actually have two bugs: if find emits a diagnostic, its result
+	     should be nonzero.  However, not having an entry for a predicate
+	     will not affect the output (just the performance) so I think it
+	     would be confusing to exit with a nonzero status.
+	  */
+	  error (0, 0,
+		 _("warning: there is no entry in the predicate evaluation "
+		   "cost table for predicate %s; please report this as a bug"),
+		 p->p_name);
 	  inherent_cost = NeedsUnknown;
 	}
     }
