@@ -37,6 +37,9 @@
 #include "error.h"
 #include "verify.h"
 #include "fdleak.h"
+#include "dircallback.h"
+#include "xalloc.h"
+#include "save-cwd.h"
 
 
 #if ENABLE_NLS
@@ -334,6 +337,26 @@ check_nofollow (void)
 #endif
 
 
+static int
+exec_cb (void *context)
+{
+  struct exec_val *execp = context;
+  bc_do_exec (&execp->ctl, &execp->state);
+  return 0;
+}
+
+static void
+do_exec (struct exec_val *execp)
+{
+  run_in_dir (execp->wd_for_exec, exec_cb, execp);
+  if (execp->wd_for_exec != initial_wd)
+    {
+      free_cwd (execp->wd_for_exec);
+      free (execp->wd_for_exec);
+      execp->wd_for_exec = NULL;
+    }
+}
+
 
 /* Examine the predicate list for instances of -execdir or -okdir
  * which have been terminated with '+' (build argument list) rather
@@ -341,14 +364,14 @@ check_nofollow (void)
  * have no effect if there are no arguments waiting).
  */
 static void
-do_complete_pending_execdirs (struct predicate *p, int dir_fd)
+do_complete_pending_execdirs (struct predicate *p)
 {
   if (NULL == p)
     return;
 
   assert (state.execdirs_outstanding);
 
-  do_complete_pending_execdirs (p->pred_left, dir_fd);
+  do_complete_pending_execdirs (p->pred_left);
 
   if (pred_is (p, pred_execdir) || pred_is(p, pred_okdir))
     {
@@ -363,20 +386,20 @@ do_complete_pending_execdirs (struct predicate *p, int dir_fd)
 	  if (execp->state.todo)
 	    {
 	      /* There are not-yet-executed arguments. */
-	      bc_do_exec (&execp->ctl, &execp->state);
+	      do_exec (execp);
 	    }
 	}
     }
 
-  do_complete_pending_execdirs (p->pred_right, dir_fd);
+  do_complete_pending_execdirs (p->pred_right);
 }
 
 void
-complete_pending_execdirs (int dir_fd)
+complete_pending_execdirs (void)
 {
   if (state.execdirs_outstanding)
     {
-      do_complete_pending_execdirs (get_eval_tree(), dir_fd);
+      do_complete_pending_execdirs (get_eval_tree());
       state.execdirs_outstanding = false;
     }
 }
@@ -418,6 +441,36 @@ complete_pending_execs (struct predicate *p)
   complete_pending_execs (p->pred_right);
 }
 
+void
+record_initial_cwd (void)
+{
+  initial_wd = xmalloc (sizeof (*initial_wd));
+  if (0 != save_cwd (initial_wd))
+    {
+      error (EXIT_FAILURE, errno,
+	     _("failed to save initial working directory"));
+    }
+}
+
+static void
+cleanup_initial_cwd (void)
+{
+  if (0 == restore_cwd (initial_wd))
+    {
+      free_cwd (initial_wd);
+      free (initial_wd);
+      initial_wd = NULL;
+    }
+  else
+    {
+      /* since we may already be in atexit, die with _exit(). */
+      error (0, errno,
+	     _("failed to restore initial working directory"));
+      _exit (EXIT_FAILURE);
+    }
+}
+
+
 static void
 traverse_tree (struct predicate *tree,
 			  void (*callback)(struct predicate*))
@@ -470,7 +523,7 @@ cleanup (void)
   if (eval_tree)
     {
       traverse_tree (eval_tree, complete_pending_execs);
-      complete_pending_execdirs (get_current_dirfd());
+      complete_pending_execdirs ();
     }
 
   /* Close ouptut files and NULL out references to them. */
@@ -478,6 +531,8 @@ cleanup (void)
   if (eval_tree)
     traverse_tree (eval_tree, undangle_file_pointers);
 
+  cleanup_initial_cwd ();
+
   if (fd_leak_check_is_enabled ())
     {
       complain_about_leaky_fds ();
@@ -488,53 +543,6 @@ cleanup (void)
     nonfatal_nontarget_file_error (errno, "standard output");
 }
 
-/* Savannah bug #16378 manifests as an assertion failure in pred_type()
- * when an NFS server returns st_mode with value 0 (of course the stat(2)
- * system call is itself returning 0 in this case).
- */
-#undef DEBUG_SV_BUG_16378
-#if defined DEBUG_SV_BUG_16378
-static int hook_fstatat (int fd, const char *name, struct stat *p, int flags)
-{
-  static int warned = 0;
-
-  if (!warned)
-    {
-      /* No use of _() here; no point asking translators to translate a debug msg */
-      error (0, 0,
-	     "WARNING: some debug code is enabled for Savannah bug #16378; "
-	     "this should not occur in released versions of findutils!");
-      warned = 1;
-    }
-
-  if (0 == strcmp (name, "./mode0file")
-      || 0 == strcmp (name, "mode0file"))
-    {
-      time_t now = time (NULL);
-      long day = 86400;
-
-      p->st_rdev = 0;
-      p->st_dev = 0x300;
-      p->st_ino = 0;
-      p->st_mode = 0;		/* SV bug #16378 */
-      p->st_nlink = 1;
-      p->st_uid = geteuid ();
-      p->st_gid = 0;
-      p->st_size = 42;
-      p->st_blksize = 32768;
-      p->st_atime = now-1*day;
-      p->st_mtime = now-2*day;
-      p->st_ctime = now-3*day;
-
-      return 0;
-    }
-  return fstatat (fd, name, p, flags);
-}
-
-# undef  fstatat
-# define fstatat (fd,name,p,flags) hook_fstatat((fd),(name),(p),(flags))
-#endif
-
 
 static int
 fallback_stat (const char *name, struct stat *p, int prev_rv)
@@ -1028,16 +1036,6 @@ set_option_defaults (struct options *p)
 }
 
 
-/* get_start_dirfd
- *
- * Returns the fd for the directory we started in.
- */
-int
-get_start_dirfd (void)
-{
-  return starting_desc;
-}
-
 /* apply_predicate
  *
  */
@@ -1064,6 +1062,15 @@ apply_predicate(const char *pathname, struct stat *stat_buf, struct predicate *p
 }
 
 
+/* is_exec_in_local_dir
+ *
+ */
+bool
+is_exec_in_local_dir (const PRED_FUNC pred_func)
+{
+  return pred_execdir == pred_func || pred_okdir == pred_func;
+}
+
 /* safely_quote_err_filename
  *
  */
