@@ -1,5 +1,5 @@
 /* Return the next element of a path.
-   Copyright (C) 1992,2005 Free Software Foundation, Inc.
+   Copyright (C) 1992, 2005, 2010 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,8 +46,7 @@ next_element (const char *new_path, int curdir_ok)
 
   if (new_path)
     {
-      if (path)
-	free (path);
+      free (path);
       end = path = strdup (new_path);
       final_colon = 0;
       return NULL;
@@ -84,16 +83,3 @@ next_element (const char *new_path, int curdir_ok)
 
   return start;
 }
-
-#ifdef TEST
-int
-main ()
-{
-  char *p;
-
-  next_element (getenv ("PATH"));
-  while (p = next_element (NULL))
-    puts (p);
-  exit (0);
-}
-#endif /* TEST */
