@@ -102,9 +102,10 @@
 #include "error.h"
 #include "closeout.h"
 #include "findutils-version.h"
+#include "xalloc.h"
 #include "progname.h"
 
-char *xmalloc PARAMS((size_t));
+
 
 /* Write out a 16-bit int, high byte first (network byte order).
  * Return true iff all went well.
